/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.awt.Point;

public class ChainField implements IChainField {
	int myNeighbourCount = 0;
	int myAtomCount = 0;
	int myPlayerID = -1;
	IChainField[] myNeighbours;
	Point myPosition = new Point();

	public void init(int x, int y, IChainField[] neighbours) {
		myPosition = new Point(x, y);
		myNeighbours = neighbours;
		myNeighbourCount = myNeighbours.length;
	}

	public boolean addAtom(int playerID) {
		return addAtom(playerID, false);
	}

	public boolean addAtom(int playerID, boolean force) {
		if (myPlayerID == -1 || myPlayerID == playerID || force) {
			if (!force && isOverloaded()) return false;		
			myPlayerID = playerID;
			myAtomCount++;
			return true;
		}
		return false;
	}

	public boolean detonate() {
		if (isOverloaded()) {
			for (int i=0; i < myNeighbours.length; i++) {
				myAtomCount--;
				myNeighbours[i].addAtom(myPlayerID, true);
			}
			if (myAtomCount <= 0) myPlayerID = -1;
			return true;
		}
		return false;
	}

	public boolean isOverloaded() {
		return (myAtomCount >= myNeighbourCount);
	}

	public int getPlayerID() {
		return myPlayerID;
	}

	public boolean isTaken() {
		return myAtomCount > 0;
	}

	public int getAtomCount() {
		return myAtomCount;
	}

	public int getNeighbourCount() {
		return myNeighbourCount;
	}

	public IChainField getNeighbour(int index) {
		return myNeighbours[index];
	}
}


