/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.awt.*;
import javax.swing.*;

public class ChainFieldPanel extends JPanel {
	boolean myExplosionMode = false;
	IChainFieldUI myOwner = null;

	public void init(IChainFieldUI owner) {
		myOwner = owner;
	}

	public void setExplosion(boolean explosionMode) {
		myExplosionMode = explosionMode;
		repaint();
	}
    public void paint(Graphics g) {
		super.paint(g);
		if (myExplosionMode) {
			drawExplosion(g);
		} else {
			myOwner.drawAtoms(g);
		}
	}

	public void drawExplosion(Graphics g) {
		Dimension size = getSize();
        int d = Math.min(size.width, size.height); 
		d = d==0?1:d;
		int r = (int)((double)d / 3.5);
        int x = size.width/2-r/2;
        int y = size.height/2-r/2;
		drawExplosion(g, x, y, (int)((double)d/1.2));
	}	
	private void drawExplosion(Graphics g, int x, int y, int s) {
		g.setColor(new Color(255, 0, 0));
		g.fillPolygon(getExplosionPolygon(x, y, s));
		g.setColor(new Color(255, 255, 0));
		g.fillPolygon(getExplosionPolygon(x+s/10, y+s/10, s/2));
		g.setColor(new Color(60, 0, 0));
		g.drawPolygon(getExplosionPolygon(x, y, s));
	}	
	private Polygon getExplosionPolygon(int x, int y, int s) {
		double f = (double)s/200;
		Polygon tmpExp = new Polygon();
		x = x - s/3; y = y - s/3;
		addPointToPolygon(tmpExp,       x,       y); addPointToPolygon(tmpExp,  75*f+x,  50*f+y); addPointToPolygon(tmpExp, 100*f+x,       y); 
		addPointToPolygon(tmpExp, 125*f+x,  50*f+y); addPointToPolygon(tmpExp, 200*f+x,       y); addPointToPolygon(tmpExp, 150*f+x,  75*f+y); 
		addPointToPolygon(tmpExp, 200*f+x, 100*f+y); addPointToPolygon(tmpExp, 150*f+x, 125*f+y); addPointToPolygon(tmpExp, 200*f+x, 200*f+y); 
		addPointToPolygon(tmpExp, 125*f+x, 150*f+y); addPointToPolygon(tmpExp, 100*f+x, 200*f+y); addPointToPolygon(tmpExp,  75*f+x, 150*f+y);
		addPointToPolygon(tmpExp,       x, 200*f+y); addPointToPolygon(tmpExp,  50*f+x, 125*f+y); addPointToPolygon(tmpExp,       x, 100*f+y); 
		addPointToPolygon(tmpExp,  50*f+x,  75*f+y); addPointToPolygon(tmpExp,       x,       y);
		return tmpExp;
	}
	private static void addPointToPolygon(Polygon target, double x, double y) {
		target.addPoint((int)x, (int)y);
	}	
	
	
}


