/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.awt.*;
import javax.swing.*;

public class ChainFieldUI  extends ChainField implements IChainFieldUI {
	private final int NEIGHBOUR_TYPE_TOP = 1;
	private final int NEIGHBOUR_TYPE_RIGHT = 2;
	private final int NEIGHBOUR_TYPE_BOTTOM = 3;
	private final int NEIGHBOUR_TYPE_LEFT = 4;

	private ChainFieldPanel myUI;
	private Color[] PlayerColor = new Color[2];

	public ChainFieldUI(ChainFieldPanel ui) {
		PlayerColor[0] = new Color(129,190,247);
		PlayerColor[1] = new Color(247,211,88);
		myUI = ui;
		myUI.init(this);
	}

	public int getX() {
		if (myUI == null) {
			return 0;		
		} else {
			return myUI.getX();
		}
	}

	public int getY() {
		if (myUI == null) {
			return 0;		
		} else {
			return myUI.getY();
		}
	}

	public Color getPlayerColor() {
		try {
			return PlayerColor[getPlayerID()];
		} catch (Exception e) {
			return new Color(255,255,255);
		}
	}

	public boolean addAtom(int playerID, boolean force) {
		boolean ret = super.addAtom(playerID, force);
		refreshUI();
		return false;
	}

	private void refreshUI() {
		if (myUI != null) myUI.repaint();
	}

	public void setExplosion(boolean value) {
		if (myUI != null) myUI.setExplosion(value);
	}

	public void drawAtoms(Graphics g) {
		drawAtoms(g, 0, 0, -1);	
	}

	public void drawAtoms(Graphics g, int offsetX, int offsetY, int offset) {
		Dimension size = myUI.getSize();
        int d = Math.min(size.width, size.height); 
		d = d==0?1:d;
		int r = (int)((double)d / 3.5);
        int x = offsetX + size.width/2-r/2;
        int y = offsetY + size.height/2-r/2;
		int o = offset;
		int c = myAtomCount;
		int i = 0;
		if (o < 0) {
			o = r; // wurde kein offset angegeben, dann entspricht der offset dem radius
		} else {
			c = getNeighbourCount(); // wenn offset, dann entspricht das maximum der kantenanzahl
		}
		if (i < c && isNeighbour(NEIGHBOUR_TYPE_TOP))    { drawAtom(g, x, y-o, r); i++; };
		if (i < c && isNeighbour(NEIGHBOUR_TYPE_RIGHT))  { drawAtom(g, x+o, y, r); i++; };
		if (i < c && isNeighbour(NEIGHBOUR_TYPE_BOTTOM)) { drawAtom(g, x, y+o, r); i++; };
		if (i < c && isNeighbour(NEIGHBOUR_TYPE_LEFT))   { drawAtom(g, x-o, y, r); i++; };
		if (i < c) drawAtom(g, x, y, r);

    }

	public IChainFieldUI getNeighbour(int index) {
		return (IChainFieldUI)super.getNeighbour(index);
	}
	
	private boolean isNeighbour(int neighbourType) {
		IChainFieldUI tmpNeighbour = null;
		for (int i = 0; i < getNeighbourCount(); i++) {
			tmpNeighbour = getNeighbour(i);
			switch (neighbourType) {
				case (NEIGHBOUR_TYPE_TOP):    { if (tmpNeighbour.getY() < getY()) return true; break; }
				case (NEIGHBOUR_TYPE_RIGHT):  { if (tmpNeighbour.getX() > getX()) return true; break; }
				case (NEIGHBOUR_TYPE_BOTTOM): { if (tmpNeighbour.getY() > getY()) return true; break; }
				case (NEIGHBOUR_TYPE_LEFT):   { if (tmpNeighbour.getX() < getX()) return true; break; }
			}
		}
		return false;
	}
	
	private void drawAtom(Graphics g, int x, int y, int s) {
		g.setColor(getPlayerColor());
		g.fillOval(x, y, s, s);
		g.setColor(new Color(0, 0, 0));
		g.drawOval(x, y, s, s);
	}

}


