/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
class ChainFieldsFactory {
	
	public IChainField[][] createChainFields(int maxX, int maxY) {
		IChainField[][] tmpChainFields = new ChainField[maxX][maxY];
		for (int y = 0; y < maxY; y++) {
			for (int x = 0; x < maxX; x++) {
				tmpChainFields[x][y] = new ChainField();
			}
		}
		ChainFieldsFactory.initChainFields(tmpChainFields, maxX, maxY);
		return tmpChainFields;
	}

	public IChainFieldUI[][] createChainFields(int maxX, int maxY, IChainPitchUI pitchUI) {
		IChainFieldUI[][] tmpChainFields = new ChainFieldUI[maxX][maxY];
		for (int y = 0; y < maxY; y++) {
			for (int x = 0; x < maxX; x++) {
				tmpChainFields[x][y] = new ChainFieldUI(pitchUI.createFieldComponent());
			}
		}
		ChainFieldsFactory.initChainFields(tmpChainFields, maxX, maxY);
		return tmpChainFields;
	}
	
	private static void initChainFields(IChainField[][] chainFields, int maxX, int maxY) {
		for (int y = 0; y < maxY; y++) {
			for (int x = 0; x < maxX; x++) {
				chainFields[x][y].init(x, y, ChainFieldsFactory.getNeighbours(chainFields, x, y, maxX, maxY));
			}
		}
	}
	
	private static IChainField[] getNeighbours(IChainField[][] chainFields, int x, int y, int maxX, int maxY) {
		int i = 0;
		int c = (x>0?1:0) + (y>0?1:0) + (x<maxX-1?1:0) + (y<maxY-1?1:0);
		//System.out.println("X: " + x + " Y: " + y + " C: " + c + " maxX: " + maxX + " maxY: " + maxY);
		IChainField[] tmpField = new ChainField[c];
		if (x > 0)       {tmpField[i] = chainFields[x-1][y  ]; i++;}
		if (y > 0)	     {tmpField[i] = chainFields[x  ][y-1]; i++;}
		if (x < maxX-1)  {tmpField[i] = chainFields[x+1][y  ]; i++;}
		if (y < maxY-1)  {tmpField[i] = chainFields[x  ][y+1]; i++;}
		return tmpField;
	}

}


