/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.util.*;

public class ChainMessage {
	private final static String PARAM_DELIMITER = "#";
	public final static String CHAIN_SOCKET_VERSION = "VERSION";
	public final static String CHAIN_SOCKET_CREATE = "CREATE";
	public final static String CHAIN_SOCKET_START = "START";
	public final static String CHAIN_SOCKET_SET = "SET";
	public final static String CHAIN_SOCKET_WIN = "WIN";
	public final static String CHAIN_SOCKET_ERR = "ERR";
	public final static String CHAIN_SOCKET_CLOSE = "CLOSE";
	
	private String myType = new String();
	private String[] myParams = null;
	private IChainChannel myChannel = null;

	public ChainMessage(String msgType, String[] params) {
		myType = msgType.toUpperCase();
		myParams = params;
	}

	public ChainMessage(IChainChannel channel) {
		myChannel = channel;
	}

	public ChainMessage(IChainChannel channel, String message) {
		myChannel = channel;
		if (message == null) return;
		int tmpStart = message.indexOf(ChainMessage.PARAM_DELIMITER);
		if (tmpStart == 0) return;
		myType = message.substring(0, tmpStart).toUpperCase();
		myParams = message.substring(tmpStart+1).split(ChainMessage.PARAM_DELIMITER);
	}

	public static ChainMessage createVersionMsg(String version) {
		String[] tmp = {version};
		return new ChainMessage(CHAIN_SOCKET_VERSION, tmp);
	}

	public static ChainMessage createCreateMsg(String name, String password, int sizeX, int sizeY) {
		String[] tmp = {name, password, String.valueOf(sizeX), String.valueOf(sizeY)};
		return new ChainMessage(CHAIN_SOCKET_CREATE, tmp);
	}

	public static ChainMessage createStartMsg(String name, int playerID) {
		String[] tmp = {name, String.valueOf(playerID)};
		return new ChainMessage(CHAIN_SOCKET_START, tmp);
	}

	public static ChainMessage createSetMsg(String name, int x, int y) {
		String[] tmp = {name, String.valueOf(x), String.valueOf(y)};
		return new ChainMessage(CHAIN_SOCKET_SET, tmp);
	}

	public static ChainMessage createSetMsg(String name, int x, int y, int playerID) {
		String[] tmp = {name, String.valueOf(x), String.valueOf(y), String.valueOf(playerID)};
		return new ChainMessage(CHAIN_SOCKET_SET, tmp);
	}

	public static ChainMessage createWinMsg(String name, int playerID) {
		String[] tmp = {name, String.valueOf(String.valueOf(playerID))};
		return new ChainMessage(CHAIN_SOCKET_WIN, tmp);
	}

	public static ChainMessage createErrorMsg(String error) {
		String[] tmp = {error};
		return new ChainMessage(CHAIN_SOCKET_ERR, tmp);
	}

	public static ChainMessage createCloseMsg(String name, String message) {
		String[] tmp = {name, message};
		return new ChainMessage(CHAIN_SOCKET_CLOSE, tmp);
	}

	public int getVersion() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_VERSION)) {
				return Integer.valueOf(myParams[0]);
			}
		} catch (Exception e) {}
		return -1;
	}

	public int getX() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_SET)) {
				return Integer.valueOf(myParams[1]);
			}
		} catch (Exception e) {}
		return -1;
	}

	public int getY() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_SET)) {
				return Integer.valueOf(myParams[2]);
			}
		} catch (Exception e) {}
		return -1;
	}

	public int getPlayerID() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_START)) {
				return Integer.valueOf(myParams[1]);
			} else if (myType.equals(ChainMessage.CHAIN_SOCKET_SET)) {
				return Integer.valueOf(myParams[3]);
			} else if (myType.equals(ChainMessage.CHAIN_SOCKET_WIN)) {
				return Integer.valueOf(myParams[1]);
			}
		} catch (Exception e) {}
		return -1;
	}

	public String getName() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_CREATE)) {
				return myParams[0];
			} else if (myType.equals(ChainMessage.CHAIN_SOCKET_SET)) {
				return myParams[0];
			} else if (myType.equals(ChainMessage.CHAIN_SOCKET_CLOSE)) {
				return myParams[0];
			} else if (myType.equals(ChainMessage.CHAIN_SOCKET_WIN)) {
				return myParams[0];
			} else if (myType.equals(ChainMessage.CHAIN_SOCKET_START)) {
				return myParams[0];
			}
		} catch (Exception e) {}
		return new String();
	}

	public String getPassword() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_CREATE)) {
				return myParams[1];
			}
		} catch (Exception e) {}
		return new String();
	}

	public int getSizeX() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_CREATE)) {
				return Integer.valueOf(myParams[2]);
			}
		} catch (Exception e) {}
		return -1;
	}

	public int getSizeY() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_CREATE)) {
				return Integer.valueOf(myParams[3]);
			}
		} catch (Exception e) {}
		return -1;
	}

	public String getError() {
		try {
			if (myType.equals(ChainMessage.CHAIN_SOCKET_ERR)) {
				return myParams[0];
			}
		} catch (Exception e) {}
		return new String();
	}

	public IChainChannel getChannel() {
		return myChannel;
	}

	public String getType() {
		return myType;
	}

	public String toString() {
		String tmpParams = new String();
		for (String param : myParams) {
			if (tmpParams.length() > 0) tmpParams = tmpParams.concat(ChainMessage.PARAM_DELIMITER);
			tmpParams = tmpParams.concat(param);
		}
		return myType + ChainMessage.PARAM_DELIMITER + tmpParams;
	}
	
}