/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.util.*;
import java.awt.*;

public class ChainPitch implements IChainPitch  {
	private IChainField[][] myChainFields;

	public ChainPitch() {}

	public ChainPitch(int maxX, int maxY) {
		myChainFields = (new ChainFieldsFactory()).createChainFields(maxX, maxY);
	}

	public boolean addAtom(int playerID, int x, int y) {
		boolean ret = myChainFields[x][y].addAtom(playerID);
		while (detonate()) {
			if (isDefeated()) break;
		}
		return ret;
	}

	public String toString() {
		String ret = new String();
		for (int y = 0; y < myChainFields[0].length; y++) {
			if (y > 0) ret = ret + "\n";
			for (int x = 0; x < myChainFields.length; x++) {
				if (x > 0) ret = ret + "   ";
				ret = ret + "C:" + myChainFields[x][y].getAtomCount() + "/" + myChainFields[x][y].getNeighbourCount() + "@";
				if (myChainFields[x][y].isTaken()) 
					ret = ret + myChainFields[x][y].getPlayerID();
				else
					ret = ret + " ";
			}
		}
		return ret;
	}

	public void setChainFields(IChainField[][] chainFields) {
		myChainFields = chainFields;
	}

	protected IChainField getField(int x, int y) {
		return myChainFields[x][y];
	}

	protected boolean detonate() {
		Vector<IChainField> tmpVector = new Vector<IChainField>();
		for (int x = 0; x < myChainFields.length; x++) {
			for (int y = 0; y < myChainFields[x].length; y++) {
				if (myChainFields[x][y].isOverloaded()) {
					tmpVector.add(myChainFields[x][y]);
				}
			}
		}
		return detonate(tmpVector);
	}

	protected boolean detonate(Vector<IChainField> fields) {
		boolean ret = false;
		for (IChainField tmpField : fields) {
			tmpField.detonate();
			ret = true;
		}
		return ret;
	}

	public boolean isDefeated() {
		int tmpPlayerID = -1; int tmpCount = 0;
		for (int x = 0; x < myChainFields.length; x++) {
			for (int y = 0; y < myChainFields[x].length; y++) {
				if (myChainFields[x][y].isTaken()) {
					if (tmpPlayerID == -1) {
						tmpPlayerID = myChainFields[x][y].getPlayerID();
					} else if (tmpPlayerID != myChainFields[x][y].getPlayerID()) {
						return false;
					}
					tmpCount++;
				}
			}
		}
		return (tmpPlayerID != -1) && (tmpCount > 1);
	}
	
}