/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.*;

public class ChainPitchUI extends ChainPitch implements IChainPitchUI {
	private ChainPitchPanel myUI;
	private int myMaxX = 0; 
	private int myMaxY = 0;

	public ChainPitchUI(int maxX, int maxY, ChainPitchPanel ui) {
		myUI = ui;
		myMaxX = maxX;
		myMaxY = maxY;
		this.setChainFields((new ChainFieldsFactory()).createChainFields(maxX, maxY, this));
		myUI.setLayout(new GridLayout(maxY, maxX));
		myUI.setDoubleBuffered(true);
		myUI.init(this);
	}
	
	protected IChainFieldUI getField(int x, int y) {
		return (IChainFieldUI)super.getField(x, y);
	}

	public ChainFieldPanel createFieldComponent() {
		ChainFieldPanel tmpUI = new ChainFieldPanel();
		myUI.add(tmpUI);
		tmpUI.setBorder(new LineBorder(new Color(22, 22, 22), 2, true));
		tmpUI.setBackground(new Color(66, 66, 66));
		tmpUI.setVisible(true);
		return tmpUI;
	}
	
	private void sleep(int value) {
		try {
			Thread.currentThread().sleep(value);
		} catch (InterruptedException e) {}
	}
	
	private int getFieldSize() {
		return myUI.getWidth() / myMaxX;
	}
	
	private void hideExplosion() {
		for (int x = 0; x < myMaxX; x++) {
			for (int y = 0; y < myMaxY; y++) {
				getField(x, y).setExplosion(false);
			}
		}
	}

	protected boolean detonate(Vector<IChainField> fields) {
		boolean ret = false;
		if (fields.size() > 0) {
			for (IChainField tmpField : fields) {
				((IChainFieldUI)tmpField).setExplosion(true);
			}
			for (int i=10; i <= getFieldSize()*0.8; i+=8) {
				myUI.setFields(fields, i);
				myUI.repaint();
				sleep(20);
			}
			myUI.setFields(null, -1);
			ret = super.detonate(fields);
			hideExplosion();
			sleep(200);
		}
		return ret;
	}

	public Point translatePos(Point pos) {
        Dimension size = myUI.getSize();
		int x = -1; int y = -1;
		try {
			x = pos.x / (size.width / myMaxX);
			y = pos.y / (size.height / myMaxY);
		} catch (Exception e) {}
		return new Point(x, y);
	}
	
}