/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.util.*;
import java.awt.*;

public class ChainSession implements IChainSession {
	String myName;
	String myPassword;
	int mySizeX;
	int mySizeY;
	IChainPitch myChainPitch;
	boolean myStartedFlag = false;
	Vector<IChainChannel> myChannelList = null;
	int myCurrentPlayerID = 0;
	
	public ChainSession(String name, String password, int sizeX, int sizeY) {
		myName = name;
		myPassword = password;
		mySizeX = sizeX;
		mySizeY = sizeY;
		myChainPitch = new ChainPitch(sizeX, sizeY);
		myChannelList = new Vector<IChainChannel>();
	}

	public boolean isPasswordValid(String password) {
		return myPassword.equals(password);
	}

	public boolean isSizeValid(int sizeX, int sizeY) {
		return (mySizeX == sizeX && mySizeY == sizeY);
	}

	public void start() {
		myStartedFlag = true;
		Iterator tmpItr = myChannelList.iterator();
		while(tmpItr.hasNext()) {
			IChainChannel tmpChannel = (IChainChannel)tmpItr.next();
			tmpChannel.send(ChainMessage.createStartMsg(myName, getPlayerID(tmpChannel)));
		}
	}

	public void close(String message) {
		myStartedFlag = false;
		Iterator tmpItr = myChannelList.iterator();
		while(tmpItr.hasNext()) {
			IChainChannel tmpChannel = (IChainChannel)tmpItr.next();
			tmpChannel.send(ChainMessage.createCloseMsg(myName, message));
		}
	}

	public boolean isStarted() {
		return myStartedFlag;
	}

	public boolean checkCurrentPlayer(IChainChannel playerChannel) {
		if (isStarted()) {
			return (getPlayerID(playerChannel) == myCurrentPlayerID);
		}
		return false;
	}

	public void shiftPlayer() {
		if (isStarted()) {
			myCurrentPlayerID++;
			if (myCurrentPlayerID >= getPlayerCount()) myCurrentPlayerID = 0;
		}	
	}

	public IChainPitch getPitch() {
		return myChainPitch;
	}

	public boolean addPlayer(IChainChannel playerChannel) {
		myChannelList.add(playerChannel);
		return true;
	}

	public int getPlayerCount() {
		try {
			return myChannelList.size();
		} catch(Exception e) {
			return -1;
		}
	}

	public int getPlayerID(IChainChannel playerChannel) {
		try {
			return myChannelList.indexOf(playerChannel);
		} catch(Exception e) {
			return -1;
		}
	}

	public String getName() {
		return myName;
	}

	public boolean send(ChainMessage message) {
		boolean ret = true;
		Iterator tmpItr = myChannelList.iterator();
		while(tmpItr.hasNext()) {
			IChainChannel tmpChannel = (IChainChannel)tmpItr.next();
			ret = tmpChannel.send(message);
		}
		return ret;
	}
	
	
}