/**
 * Copyright by Manuel Siekmann (makasy.com)
 * The License.txt file describes the conditions under which this software may be distributed.
 * @author Manuel Siekmann
 * @version 1.0
 */
import java.io.*;
import java.net.*;
import java.nio.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.nio.charset.*;
import java.nio.channels.*;

public class ModulTest extends ChainPitch implements Runnable, IChainChannelListener {
	private static final int MAX_X = 6;
	private static final int MAX_Y = 5;
	private final static String HOSTNAME = "127.0.0.1";
	private final static int PORT = 107;
	private String myName = new String();
	private String myPassword = new String();
	private boolean myPendingAtomFlag = false;
	private boolean myReadyFlag = false;
	private boolean myStartedFlag = false;
	private boolean myErrorFlag = false;
	private boolean myWinFlag = false;
	private int myPlayerID = -1;
	private int myLastPlayerID = -1;
	IChainChannel myChannel = null;
	SocketChannel mySocketChannel = null;
	Thread myThread = null;

	public ModulTest(String name, String password) {
		super(MAX_X, MAX_Y);
		myName = name;
		myPassword = password;
		connect();
	}

	public void onReceived(ChainMessage e) {
		String tmpType = e.getType();
		//System.out.println("R:" + myPlayerID + ">" + e.toString());
		if (tmpType.equals(ChainMessage.CHAIN_SOCKET_VERSION)) {
			myChannel.send(ChainMessage.createCreateMsg(myName, myPassword, MAX_X, MAX_Y));
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_SET)) {
			addAtom(e.getPlayerID(), e.getX(), e.getY());
			if (e.getPlayerID() == getPlayerID()) setPendingAtom(false);
			setLastPlayerID(e.getPlayerID());
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_CREATE)) {
			setReady(true);
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_START)) {
			setPlayerID(e.getPlayerID());
			setReady(true);
			setStarted(true);
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_CLOSE)) {
			setReady(false);
			myThread = null;
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_ERR)) {
			setError(true);
			setPendingAtom(false);
		} else if (tmpType.equals(ChainMessage.CHAIN_SOCKET_WIN)) {
			setWin(true);
		}
		
	}
	
	public void onConnected(ChainMessage e) {
	}
	
	public void onClosed(ChainMessage e) {
		setReady(false);
	}

	public synchronized void addAtom(int x, int y) {
		setPendingAtom(true);
		myChannel.send(ChainMessage.createSetMsg(myName, x, y));

	private synchronized void setPlayerID(int value) {
		myPlayerID = value;
	}

	private synchronized void setLastPlayerID(int value) {
		myLastPlayerID = value;
	}

	private synchronized void setStarted(boolean value) {
		myStartedFlag = value;
	}

	private synchronized void setReady(boolean value) {
		myReadyFlag = value;
	}

	private synchronized void setError(boolean value) {
		myErrorFlag = value;
	}

	private synchronized void setWin(boolean value) {
		myWinFlag = value;
	}

	private synchronized void setPendingAtom(boolean value) {
		myPendingAtomFlag = value;
	}

	public synchronized boolean isStarted() {
		return myStartedFlag;
	}

	public synchronized boolean isReady() {
		return myReadyFlag;
	}

	public synchronized boolean isError() {
		return myErrorFlag;
	}

	private synchronized boolean isWin() {
		return myWinFlag;
	}

	public synchronized boolean isPendingAtom() {
		return myPendingAtomFlag;
	}

	public synchronized int getPlayerID() {
		return myPlayerID;
	}

	public synchronized int getLastPlayerID() {
		return myLastPlayerID;
	}
	
	private boolean connect() {
		try {
			mySocketChannel = SocketChannel.open();
			mySocketChannel.configureBlocking(false);		
			mySocketChannel.connect(new InetSocketAddress(HOSTNAME, PORT));
			myThread = new Thread(this);
			myThread.start();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return true;
	}
	
	public void run() {
		try {
			Selector tmpSel = Selector.open();
			SelectionKey tmpConnectKey = mySocketChannel.register(tmpSel, SelectionKey.OP_CONNECT);
			while (true) {
				try {
					tmpSel.select();
					for (Iterator<SelectionKey> tmpKeyIterator = tmpSel.selectedKeys().iterator(); tmpKeyIterator.hasNext();) {
						SelectionKey tmpKey = tmpKeyIterator.next();
						tmpKeyIterator.remove();
						if (tmpKey.isConnectable()) {
							if(mySocketChannel.isConnectionPending()) mySocketChannel.finishConnect();
							myChannel = new ChainChannel(tmpSel, mySocketChannel, this);
						} else {
							IChainChannel tmpChainChannel = null;
							try {
								tmpChainChannel = (IChainChannel)(tmpKey.attachment());
							} catch (Exception e) {
								e.printStackTrace();
							}
							if (tmpChainChannel != null) {
								tmpChainChannel.process();
							}
						}
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static long getTime() {
		return (new Date().getTime());
	}

	public static void sleep(long millis) {
		try {
			Thread.currentThread().sleep(millis);
		} catch(Exception e) {
			//...
		}
	}
	public static boolean addAtom(ModulTest player, int x, int y) {
		return addAtom(player, x, y, true);		
	}

	public static boolean addAtom(ModulTest player, int x, int y, boolean waitForSlot) {
		if (player == null) return false;
		player.setError(false);
		if (waitForSlot) {
			if (player.isReady() == false) return false;
			if (!wait(player, 2000)) return false;
			long tmpStart = getTime();
			while (player.getLastPlayerID() == player.getPlayerID()) {
				if  ((getTime() - tmpStart) > 2000) { 
					return false; 
				}
				sleep(10);
			}
		}
		player.addAtom(x, y);
		if (waitForSlot) {
			if (!wait(player, 2000)) return false;
		} else {
			if (!wait(player, false, false, 2000)) return false;
		}
		return !player.isError();
	}
	
	public static boolean wait(ModulTest player) {
		return wait(player, 2000);		
	}
	
	public static boolean wait(ModulTest player, int delay) {
		return wait(player, true, false, delay);		
	}

	public static boolean wait(ModulTest player, boolean started, boolean win, int delay) {
		long tmpStart = getTime();
		while (!(player.isReady() || (win && player.isWin()))) {
			if  ((getTime() - tmpStart) > delay) {
				System.out.println("TIMEOUT: isReady()");
				return false; 
			}
			sleep(10);
		}
		if (started) {
			while (!player.isStarted()) {
				if  ((getTime() - tmpStart) > delay) {
					System.out.println("TIMEOUT: isStarted()");
					return false; 
				}
				sleep(10);
			}
		}
		if (win) {
			while (!player.isWin()) {
				if  ((getTime() - tmpStart) > delay) {
					System.out.println("TIMEOUT: isWin()");
					return false; 
				}
				sleep(10);
			}
		}
		while (player.isPendingAtom()) {
			if  ((getTime() - tmpStart) > delay) {
				System.out.println("TIMEOUT: isPending()");
				return false; 
			}
			sleep(10);
		}
		return true;
	}

	public static boolean simulateSimpleGame(ModulTest player1, ModulTest player2) {
		wait(player1);
		wait(player2);
		if (addAtom(player1, 0, 0)) {
			if (player1.isDefeated() || player2.isDefeated()) return false;
			if (addAtom(player2, 0, 1)) {
				if (player1.isDefeated() || player2.isDefeated()) return false;
				if (addAtom(player1, 0, 0)) {
					wait(player1, false, true, 2000);
					wait(player2, false, true, 2000);		
					if (player1.isDefeated() && player2.isDefeated()) return true;
				}
			}
		}
		return false;
	}

	public static boolean simulateComplexGame(ModulTest player1, ModulTest player2) {
		wait(player1, 2000);
		wait(player2, 2000);
		int playerID = 0;
		int[][] complexSteps = {{4,1},{0,4},{1,3},{5,0},{4,0},{5,0},{5,2},{5,1},{4,1},{5,1},{5,1},{4,3},{4,2},{4,0},{2,0},{5,4},{2,0},{0,0},{2,0},{5,0},{2,1},{2,0},{2,1},{2,0},{1,3},{3,3},{1,3},{3,1},{3,4},{5,2},{3,4},{5,3},{3,4},{5,1},{5,0},{4,0},{5,2},{4,4},{5,2},{4,4},{5,2},{5,4},{1,3},{3,1},{0,3},{1,4}};
		for (int i = 0; i < complexSteps.length; i++) {
			if (player1.isDefeated() || player2.isDefeated()) return false;
			if (addAtom(playerID==0?player1:player2, complexSteps[i][0], complexSteps[i][1]) == false) return false;
			playerID = playerID==0?1:0;
		}
		wait(player1, false, true, 1000);
		wait(player2, false, true, 1000);		
		if (player1.isDefeated() && player2.isDefeated()) return true;
		return false;
	}

	public static boolean test_parallel_game() {
		boolean ret = false;
		ModulTest[] tmpPlayer1 = new ModulTest[2];
		ModulTest[] tmpPlayer2 = new ModulTest[2];
		for (int i = 0; i<2; i++) {
			tmpPlayer1[i] = new ModulTest("parallel-" + i, "test");
			wait(tmpPlayer1[i], false, false, 2000);
			tmpPlayer2[i] = new ModulTest("parallel-" + i, "test");			
		}
		ret = (simulateSimpleGame(tmpPlayer1[0], tmpPlayer2[0]) && simulateSimpleGame(tmpPlayer1[1], tmpPlayer2[1]));
		for (int i = 0; i<2; i++) {
			tmpPlayer1[i] = null;
			tmpPlayer2[i] = null;
		}
		return ret;
	}

	public static boolean test_100_player() {
		ModulTest[] tmpPlayer = new ModulTest[100];
		boolean ret = true;
		for (int i = 0; i<100; i+=2) {
			tmpPlayer[i] = new ModulTest("100 Spieler in 50 Sitzungen " + i, "test");
			if (wait(tmpPlayer[i], false, false, 2000) == false) System.out.println("Timeout...");
			tmpPlayer[i+1] = new ModulTest("100 Spieler in 50 Sitzungen " + i, "test");			
			wait(tmpPlayer[i+1], false, false, 2000);
		}
		// Spielzug 1
		for (int i = 0; i <100; i+=2) {
			if (addAtom(tmpPlayer[i], 0, 0)) {
				if (tmpPlayer[i].isDefeated() || tmpPlayer[i+1].isDefeated()) ret = false;
				if (!ret) break;
			}
		}
		// Spielzug 2
		for (int i = 0; i <100; i+=2) {
			if (addAtom(tmpPlayer[i+1], 0, 1)) {
				if (tmpPlayer[i].isDefeated() || tmpPlayer[i+1].isDefeated()) ret = false;
				if (!ret) break;
			}
		}
		// Spielzug 3
		for (int i = 0; i <100; i+=2) {
			if (addAtom(tmpPlayer[i], 0, 0)) {
				if (!wait(tmpPlayer[i], false, true, 1000)) System.out.println("TIMEOUT!");
				if (!wait(tmpPlayer[i+1], false, true, 1000)) System.out.println("TIMEOUT!");	
				if (!tmpPlayer[i].isDefeated() || !tmpPlayer[i+1].isDefeated()) ret = false;
				if (!ret) break;
			}
		}
		return ret;		
	}

	public static boolean test_simple_game() {
		ModulTest tmpPlayer1 = new ModulTest("simple", "---"); wait(tmpPlayer1, false, false, 2000);
		ModulTest tmpPlayer2 = new ModulTest("simple", "---");
		return simulateSimpleGame(tmpPlayer1, tmpPlayer2);
	}

	public static boolean test_complex_game() {
		ModulTest tmpPlayer1 = new ModulTest("complex", "---"); wait(tmpPlayer1, false, false, 2000);
		ModulTest tmpPlayer2 = new ModulTest("complex", "---");
		return simulateComplexGame(tmpPlayer1, tmpPlayer2);
	}

	public static boolean test_input_validator() {
		boolean ret = true;
		ModulTest tmpPlayer1 = new ModulTest("validator", "---"); 
		wait(tmpPlayer1, false, false, 2000);
		
		addAtom(tmpPlayer1, 0, 0, false);
		if (!tmpPlayer1.isError()) ret = false; // noch nicht gestartet
		
		ModulTest tmpPlayer2 = new ModulTest("validator", "---");
		wait(tmpPlayer1);
		wait(tmpPlayer2);
		
		addAtom(tmpPlayer1, 0, 0, false);
		if (tmpPlayer1.isError()) ret = false; // ist legal
		
		addAtom(tmpPlayer2, 0, 0, false);
		if (!tmpPlayer2.isError()) ret = false; // bereits belegt
		
		addAtom(tmpPlayer2, 1, 1, false);
		if (tmpPlayer2.isError()) ret = false; // ist legal
		
		addAtom(tmpPlayer2, 3, 3, false);
		if (!tmpPlayer2.isError()) ret = false; // reihenfolge
		
		return ret;
	}

	public static void main(String[] args) {
		System.out.print("Einfaches Spiel: ");
		if (test_simple_game()) {
			System.out.print("erfolgreich\n");
		} else {
			System.out.print("fehlerhaft\n");
		}
		System.out.print("Komplexes Spiel: ");
		if (test_complex_game()) {
			System.out.print("erfolgreich\n");
		} else {
			System.out.print("fehlerhaft\n");
		}
		System.out.print("Parallele Sitzungen: ");
		if (test_parallel_game()) {
			System.out.print("erfolgreich\n");
		} else {
			System.out.print("fehlerhaft\n");
		}
		System.out.print("Validierung der Eingaben: ");
		if (test_input_validator()) {
			System.out.print("erfolgreich\n");
		} else {
			System.out.print("fehlerhaft\n");
		}
		System.out.print("100 Spieler in 50 Sitzungen: ");
		if (test_100_player()) {
			System.out.print("erfolgreich\n");
		} else {
			System.out.print("fehlerhaft\n");
		}
		System.exit(0);

	}


	
	
	
}





